@@ -1,7 +1,7 @@
 ###############################################################################
 # R (https://r-project.org/) Numeric Methods for Optimization of Portfolios
 #
-# Copyright (c) 2004-2018 Brian G. Peterson, Peter Carl, Ross Bennett, Kris Boudt
+# Copyright (c) 2004-2023 Brian G. Peterson, Peter Carl, Ross Bennett, Kris Boudt, Xinran Zhao
 #
 # This library is distributed under the terms of the GNU Public License (GPL)
 # for full details see the file COPYING
@@ -81,7 +81,7 @@ extractStats <- function (object, prefix=NULL, ...){
 }
 
 #' @method extractStats optimize.portfolio.DEoptim
-#' @S3method extractStats optimize.portfolio.DEoptim
+
 #' @export 
 extractStats.optimize.portfolio.DEoptim <- function(object, prefix=NULL, ...) {
   if(!inherits(object, "optimize.portfolio.DEoptim")) stop("object must be of class optimize.portfolio.DEoptim")
@@ -115,7 +115,7 @@ extractStats.optimize.portfolio.DEoptim <- function(object, prefix=NULL, ...) {
 }
 
 #' @method extractStats optimize.portfolio.ROI
-#' @S3method extractStats optimize.portfolio.ROI
+
 #' @export 
 extractStats.optimize.portfolio.ROI <- function(object, prefix=NULL, ...) {
   if(!inherits(object, "optimize.portfolio.ROI")) stop("object must be of class optimize.portfolio.ROI")
@@ -131,8 +131,25 @@ extractStats.optimize.portfolio.ROI <- function(object, prefix=NULL, ...) {
   return(result)
 }
 
+#' @method extractStats optimize.portfolio.CVXR
+
+#' @export 
+extractStats.optimize.portfolio.CVXR <- function(object, prefix=NULL, ...) {
+  if(!inherits(object, "optimize.portfolio.CVXR")) stop("object must be of class optimize.portfolio.CVXR")
+  trow <- c(object$out, object$weights)
+  objmeas <- extractObjectiveMeasures(object)
+  objnames <- names(objmeas)
+  obj <- unlist(objmeas)
+  result <- c(obj, trow)
+  rnames<-c(objnames, 'out', paste('w', names(object$weights), sep='.'))
+  #print(result)
+  #print(rnames)
+  names(result)<-rnames
+  return(result)
+}
+
 #' @method extractStats optimize.portfolio.pso
-#' @S3method extractStats optimize.portfolio.pso
+
 #' @export 
 extractStats.optimize.portfolio.pso <- function(object, prefix=NULL, ...){
   if(!inherits(object, "optimize.portfolio.pso")) stop("object must be of class optimize.portfolio.pso")
@@ -203,7 +220,7 @@ extractStats.optimize.portfolio.pso <- function(object, prefix=NULL, ...){
 }
 
 #' @method extractStats optimize.portfolio.GenSA
-#' @S3method extractStats optimize.portfolio.GenSA
+
 #' @export 
 extractStats.optimize.portfolio.GenSA <- function(object, prefix=NULL, ...) {
   if(!inherits(object, "optimize.portfolio.GenSA")) stop("object must be of class optimize.portfolio.GenSA")
@@ -221,7 +238,7 @@ extractStats.optimize.portfolio.GenSA <- function(object, prefix=NULL, ...) {
 }
 
 #' @method extractStats optimize.portfolio.invol
-#' @S3method extractStats optimize.portfolio.invol
+
 #' @export 
 extractStats.optimize.portfolio.invol <- function(object, prefix=NULL, ...) {
   if(!inherits(object, "optimize.portfolio.invol")) stop("object must be of class optimize.portfolio.invol")
@@ -236,7 +253,7 @@ extractStats.optimize.portfolio.invol <- function(object, prefix=NULL, ...) {
 }
 
 #' @method extractStats optimize.portfolio.eqwt
-#' @S3method extractStats optimize.portfolio.eqwt
+
 #' @export 
 extractStats.optimize.portfolio.eqwt <- function(object, prefix=NULL, ...) {
   if(!inherits(object, "optimize.portfolio.eqwt")) stop("object must be of class optimize.portfolio.eqwt")
@@ -251,7 +268,7 @@ extractStats.optimize.portfolio.eqwt <- function(object, prefix=NULL, ...) {
 }
 
 #' @method extractStats optimize.portfolio.rebalancing
-#' @S3method extractStats optimize.portfolio.rebalancing
+
 #' @export 
 extractStats.optimize.portfolio.rebalancing <- function(object, prefix=NULL, ...) {
   if(!inherits(object, "optimize.portfolio.rebalancing")) stop("object must be of class optimize.portfolio.rebalancing")
@@ -295,7 +312,7 @@ extractStatsRegime <- function(object, prefix=NULL){
 }
 
 #' @method extractStats optimize.portfolio.parallel
-#' @S3method extractStats optimize.portfolio.parallel
+
 #' @export
 extractStats.optimize.portfolio.parallel <- function(object,prefix=NULL,...) {
     resultlist<-object
@@ -311,7 +328,7 @@ extractStats.optimize.portfolio.parallel <- function(object,prefix=NULL,...) {
 }
 
 #' @method extractStats optimize.portfolio.random
-#' @S3method extractStats optimize.portfolio.random
+
 #' @export
 extractStats.optimize.portfolio.random <- function(object, prefix=NULL, ...){
 # This just flattens the $random_portfolio_objective_results part of the object
@@ -344,7 +361,7 @@ extractStats.optimize.portfolio.random <- function(object, prefix=NULL, ...){
 }
 
 #' @method extractStats opt.list
-#' @S3method extractStats opt.list
+
 #' @export
 extractStats.opt.list <- function(object, ...){
   # get the stats of each optimization in a list
@@ -357,7 +374,7 @@ extractStats.opt.list <- function(object, ...){
 }
 
 #' @method extractStats opt.rebal.list
-#' @S3method extractStats opt.rebal.list
+
 #' @export
 extractStats.opt.rebal.list <- function(object, ...){
   # get the stats of each optimization in a list
@@ -385,7 +402,7 @@ extractWeights <- function (object, ...){
 }
 
 #' @method extractWeights optimize.portfolio
-#' @S3method extractWeights optimize.portfolio
+
 #' @export
 extractWeights.optimize.portfolio <- function(object, ...){
   if(!inherits(object, "optimize.portfolio")){
@@ -395,7 +412,7 @@ extractWeights.optimize.portfolio <- function(object, ...){
 }
 
 #' @method extractWeights optimize.portfolio.rebalancing
-#' @S3method extractWeights optimize.portfolio.rebalancing
+
 #' @export
 extractWeights.optimize.portfolio.rebalancing <- function(object, ...){
   if(!inherits(object, "optimize.portfolio.rebalancing")){
@@ -417,14 +434,14 @@ extractWeights.optimize.portfolio.rebalancing <- function(object, ...){
 }
 
 #' @method extractWeights summary.optimize.portfolio.rebalancing
-#' @S3method extractWeights summary.optimize.portfolio.rebalancing
+
 #' @export
 extractWeights.summary.optimize.portfolio.rebalancing <- function(object, ...){
   object$weights
 }
 
 #' @method extractWeights opt.list
-#' @S3method extractWeights opt.list
+
 #' @export
 extractWeights.opt.list <- function(object, ...){
   # get the optimal weights of each optimization in a list
@@ -454,7 +471,7 @@ extractWeights.opt.list <- function(object, ...){
 }
 
 #' @method extractWeights opt.rebal.list
-#' @S3method extractWeights opt.rebal.list
+
 #' @export
 extractWeights.opt.rebal.list <- function(object, ...){
   # get the optimal weights of each optimization in a list
@@ -483,7 +500,8 @@ extractObjectiveMeasures <- function(object){
 }
 
 #' @method extractObjectiveMeasures optimize.portfolio
-#' @S3method extractObjectiveMeasures optimize.portfolio
+
+#' @export
 extractObjectiveMeasures.optimize.portfolio <- function(object){
   if(!inherits(object, "optimize.portfolio")) stop("object must be of class 'optimize.portfolio'")
   # objective measures returned as $objective_measures for all other solvers
@@ -492,7 +510,8 @@ extractObjectiveMeasures.optimize.portfolio <- function(object){
 }
 
 #' @method extractObjectiveMeasures optimize.portfolio.rebalancing
-#' @S3method extractObjectiveMeasures optimize.portfolio.rebalancing
+
+#' @export
 extractObjectiveMeasures.optimize.portfolio.rebalancing <- function(object){
   if(!inherits(object, "optimize.portfolio.rebalancing")) stop("object must be of class 'optimize.portfolio.rebalancing'")
   
@@ -553,13 +572,15 @@ extractObjRegime <- function(object){
 }
 
 #' @method extractObjectiveMeasures summary.optimize.portfolio.rebalancing
-#' @S3method extractObjectiveMeasures summary.optimize.portfolio.rebalancing
+
+#' @export
 extractObjectiveMeasures.summary.optimize.portfolio.rebalancing <- function(object){
   object$objective_measures
 }
 
 #' @method extractObjectiveMeasures opt.list
-#' @S3method extractObjectiveMeasures opt.list
+
+#' @export
 extractObjectiveMeasures.opt.list <- function(object){
   # The idea is that these portfolios opt.list may have different objectives.
   # Need a function to evaluate *all* objective measures for each portfolio.
@@ -660,7 +681,7 @@ extractObjectiveMeasures.opt.list <- function(object){
 }
 
 #' @method extractObjectiveMeasures opt.rebal.list
-#' @S3method extractObjectiveMeasures opt.rebal.list
+
 #' @export
 extractObjectiveMeasures.opt.rebal.list <- function(object, ...){
   # get the optimal weights of each optimization in a list
@@ -683,7 +704,7 @@ extractObjectiveMeasures.opt.rebal.list <- function(object, ...){
 #' @param object object of class \code{optimize.portfolio}
 #' @param ... passthrough parameters. Not currently used
 #' @return a list with two elements
-#' \itemize{
+#' \describe{
 #'   \item{weights: }{Optimal set of weights from the \code{optimize.portfolio} object}
 #'   \item{category_weights: }{Weights by category if category_labels are supplied in the \code{portfolio} object}
 #'   \item{group_weights: }{Weights by group if group is a constraint type}
