@@ -1,7 +1,7 @@
 ###############################################################################
 # R (https://r-project.org/) Numeric Methods for Optimization of Portfolios
 #
-# Copyright (c) 2004-2018 Brian G. Peterson, Peter Carl, Ross Bennett, Kris Boudt
+# Copyright (c) 2004-2021 Brian G. Peterson, Peter Carl, Ross Bennett, Kris Boudt
 #
 # This library is distributed under the terms of the GNU Public License (GPL)
 # for full details see the file COPYING
@@ -21,7 +21,8 @@
 #' @author Ross Bennett
 #' @rdname print.optimize.portfolio.rebalancing
 #' @method print optimize.portfolio.rebalancing
-#' @S3method print optimize.portfolio.rebalancing
+
+#' @export
 print.optimize.portfolio.rebalancing <- function(x, ..., digits=4){
   cat(rep("*", 50) ,"\n", sep="")
   cat("PortfolioAnalytics Optimization with Rebalancing\n")
@@ -98,7 +99,8 @@ summary.optimize.portfolio.rebalancing <- function(object, ...) {
 #' @seealso \code{\link{summary.optimize.portfolio.rebalancing}}
 #' @author Ross Bennett
 #' @method print summary.optimize.portfolio.rebalancing
-#' @S3method print summary.optimize.portfolio.rebalancing
+
+#' @export
 print.summary.optimize.portfolio.rebalancing <- function(x, ..., digits=4){
   cat(rep("*", 50) ,"\n", sep="")
   cat("PortfolioAnalytics Optimization with Rebalancing\n")
@@ -147,7 +149,8 @@ print.summary.optimize.portfolio.rebalancing <- function(x, ..., digits=4){
 #' @seealso \code{\link{portfolio.spec}}
 #' @author Ross Bennett
 #' @method print portfolio
-#' @S3method print portfolio
+
+#' @export
 print.portfolio <- function(x, ...){
   if(!is.portfolio(x)) stop("object passed in is not of class 'portfolio'")
   
@@ -345,7 +348,8 @@ summary.portfolio <- function(object, ...){
 #' @param \dots any other passthru parameters
 #' @author Ross Bennett
 #' @method print constraint
-#' @S3method print constraint
+
+#' @export
 print.constraint <- function(x, ...){
   print.default(x, ...)
 }
@@ -361,7 +365,8 @@ print.constraint <- function(x, ...){
 #' @author Ross Bennett
 #' @rdname print.optimize.portfolio
 #' @method print optimize.portfolio.ROI
-#' @S3method print optimize.portfolio.ROI
+
+#' @export
 print.optimize.portfolio.ROI <- function(x, ..., digits=4){
   cat(rep("*", 35) ,"\n", sep="")
   cat("PortfolioAnalytics Optimization\n")
@@ -400,9 +405,52 @@ print.optimize.portfolio.ROI <- function(x, ..., digits=4){
 }
 
 
+#' @rdname print.optimize.portfolio
+#' @method print optimize.portfolio.CVXR
+
+#' @export
+print.optimize.portfolio.CVXR <- function(x, ..., digits=4){
+  cat(rep("*", 35) ,"\n", sep="")
+  cat("PortfolioAnalytics Optimization\n")
+  cat(rep("*", 35) ,"\n", sep="")
+  
+  cat("\nCall:\n", paste(deparse(x$call), sep = "\n", collapse = "\n"), 
+      "\n\n", sep = "")
+  
+  # get optimal weights
+  cat("Optimal Weights:\n")
+  print.default(round(x$weights, digits=digits), digits=digits)
+  cat("\n")
+  
+  # get objective measures
+  objective_measures <- x$objective_measures
+  tmp_obj <- as.numeric(unlist(objective_measures))
+  names(tmp_obj) <- names(objective_measures)
+  cat("Objective Measures:\n")
+  for(i in 1:length(objective_measures)){
+    print(tmp_obj[i], digits=4)
+    cat("\n")
+    if(length(objective_measures[[i]]) > 1){
+      # This will be the case for any objective measures with risk budgets
+      for(j in 2:length(objective_measures[[i]])){
+        tmpl <- objective_measures[[i]][j]
+        cat(names(tmpl), ":\n")
+        tmpv <- unlist(tmpl)
+        names(tmpv) <- names(x$weights)
+        print.default(round(tmpv, digits=digits), digits=digits)
+        cat("\n")
+      }
+    }
+    cat("\n")
+  }
+  cat("\n")
+}
+
+
 #' @rdname print.optimize.portfolio
 #' @method print optimize.portfolio.random
-#' @S3method print optimize.portfolio.random
+
+#' @export
 print.optimize.portfolio.random <- function(x, ..., digits=4){
   cat(rep("*", 35) ,"\n", sep="")
   cat("PortfolioAnalytics Optimization\n")
@@ -443,7 +491,8 @@ print.optimize.portfolio.random <- function(x, ..., digits=4){
 
 #' @rdname print.optimize.portfolio
 #' @method print optimize.portfolio.DEoptim
-#' @S3method print optimize.portfolio.DEoptim
+
+#' @export
 print.optimize.portfolio.DEoptim <- function(x, ..., digits=4){
   cat(rep("*", 35) ,"\n", sep="")
   cat("PortfolioAnalytics Optimization\n")
@@ -484,7 +533,8 @@ print.optimize.portfolio.DEoptim <- function(x, ..., digits=4){
 
 #' @rdname print.optimize.portfolio
 #' @method print optimize.portfolio.GenSA
-#' @S3method print optimize.portfolio.GenSA
+
+#' @export
 print.optimize.portfolio.GenSA <- function(x, ..., digits=4){
   cat(rep("*", 35) ,"\n", sep="")
   cat("PortfolioAnalytics Optimization\n")
@@ -525,7 +575,8 @@ print.optimize.portfolio.GenSA <- function(x, ..., digits=4){
 
 #' @rdname print.optimize.portfolio
 #' @method print optimize.portfolio.pso
-#' @S3method print optimize.portfolio.pso
+
+#' @export
 print.optimize.portfolio.pso <- function(x, ..., digits=4){
   cat(rep("*", 35) ,"\n", sep="")
   cat("PortfolioAnalytics Optimization\n")
@@ -572,7 +623,8 @@ print.optimize.portfolio.pso <- function(x, ..., digits=4){
 #' @seealso \code{\link{summary.optimize.portfolio}}
 #' @author Ross Bennett
 #' @method print summary.optimize.portfolio
-#' @S3method print summary.optimize.portfolio
+
+#' @export
 print.summary.optimize.portfolio <- function(x, ...){
   
   cat(rep("*", 50) ,"\n", sep="")
@@ -774,7 +826,8 @@ print.summary.optimize.portfolio <- function(x, ...){
 #' @seealso \code{\link{optimize.portfolio}}
 #' @author Ross Bennett
 #' @method summary optimize.portfolio
-#' @S3method summary optimize.portfolio
+
+#' @export
 summary.optimize.portfolio <- function(object, ...){
   
   out <- list()
@@ -905,7 +958,8 @@ summary.optimize.portfolio <- function(object, ...){
 #' @seealso \code{\link{create.EfficientFrontier}}
 #' @author Ross Bennett
 #' @method print efficient.frontier
-#' @S3method print efficient.frontier
+
+#' @export
 print.efficient.frontier <- function(x, ...){
   if(!inherits(x, "efficient.frontier")) stop("object passed in is not of class 'efficient.frontier'")
   
@@ -965,7 +1019,7 @@ summary.efficient.frontier <- function(object, ..., digits=3){
 }
 
 #' @method print portfolio.list
-#' @S3method print portfolio.list
+
 #' @export
 print.portfolio.list <- function(x, ...){
   for(i in 1:length(x)){
@@ -975,7 +1029,7 @@ print.portfolio.list <- function(x, ...){
 }
 
 #' @method print opt.list
-#' @S3method print opt.list
+
 #' @export
 print.opt.list <- function(x, ...){
   for(i in 1:length(x)){
@@ -985,7 +1039,7 @@ print.opt.list <- function(x, ...){
 }
 
 #' @method print opt.rebal.list
-#' @S3method print opt.rebal.list
+
 #' @export
 print.opt.rebal.list <- function(x, ...){
   for(i in 1:length(x)){
@@ -995,7 +1049,7 @@ print.opt.rebal.list <- function(x, ...){
 }
 
 #' @method print regime.portfolios
-#' @S3method print regime.portfolios
+
 #' @export
 print.regime.portfolios <- function(x, ...){
   
@@ -1013,7 +1067,7 @@ print.regime.portfolios <- function(x, ...){
 }
 
 #' @method summary optimize.portfolio.parallel
-#' @S3method summary optimize.portfolio.parallel
+
 #' @export
 summary.optimize.portfolio.parallel <- function(object, ...){
   out <- list()
@@ -1032,7 +1086,7 @@ summary.optimize.portfolio.parallel <- function(object, ...){
 }
 
 #' @method print optimize.portfolio.parallel
-#' @S3method print optimize.portfolio.parallel
+
 #' @export
 print.optimize.portfolio.parallel <- function(x, ..., probs = c(0.025, 0.975)){
   cat(rep("*", 35) ,"\n", sep="")
