@@ -1,7 +1,7 @@
 ###############################################################################
 # R (https://r-project.org/) Numeric Methods for Optimization of Portfolios
 #
-# Copyright (c) 2004-2018 Brian G. Peterson, Peter Carl, Ross Bennett, Kris Boudt
+# Copyright (c) 2004-2021 Brian G. Peterson, Peter Carl, Ross Bennett, Kris Boudt
 #
 # This library is distributed under the terms of the GNU Public License (GPL)
 # for full details see the file COPYING
@@ -144,7 +144,7 @@ set.portfolio.moments_v1 <- function(R, constraints, momentargs=NULL,...){
 #' Set portfolio moments for use by lower level optimization functions. Currently
 #' three methods for setting the moments are available
 #' 
-#' \itemize{
+#' \describe{
 #'   \item{sample: }{sample estimates are used for the moments}
 #'   \item{boudt: }{estimate the second, third, and fourth moments using a 
 #'   statistical factor model based on the work of Kris Boudt.}
@@ -280,7 +280,8 @@ set.portfolio.moments <- set.portfolio.moments_v2 <- function(R,
                ) # end nested switch on method 
              }, # end switch on var, sd, StdDev
              mVaR =,
-             VaR = {
+             VaR = ,
+             EQS = {
                switch(method,
                       sample = {
                         if(is.null(momentargs$mu)) momentargs$mu = matrix( as.vector(apply(tmpR, 2, 'mean')), ncol=1);
@@ -414,7 +415,8 @@ portfolio.moments.boudt <- function(R, portfolio, momentargs=NULL, k=1, ...){
                if(is.null(momentargs$sigma)) momentargs$sigma = extractCovariance(fit)
              },
              mVaR =,
-             VaR = {
+             VaR = ,
+             EQS = {
                if(is.null(momentargs$mu)) momentargs$mu = matrix( as.vector(apply(R,2,'mean')),ncol=1)
                if(is.null(momentargs$sigma)) momentargs$sigma = extractCovariance(fit)
                if(is.null(momentargs$m3)) momentargs$m3 = extractCoskewness(fit)
@@ -499,7 +501,8 @@ portfolio.moments.bl <- function(R, portfolio, momentargs=NULL, P, Mu=NULL, Sigm
                if(is.null(momentargs$sigma)) momentargs$sigma = B$BLSigma
              },
              mVaR =,
-             VaR = {
+             VaR = ,
+             EQS = {
                if(is.null(momentargs$mu)) momentargs$mu = B$BLMu
                if(is.null(momentargs$sigma)) momentargs$sigma = B$BLSigma
                if(is.null(momentargs$m3)) momentargs$m3 = PerformanceAnalytics::M3.MM(R)
