@@ -1,7 +1,11 @@
 #' @import xts
 #' @import zoo
+#' @import ROI.plugin.symphony
+#' @import GenSA
+#' @import mco
 #' @import PerformanceAnalytics
 #' @import foreach
+#' @importFrom pso        lines plot points
 #' @importFrom graphics   abline axis barplot box hist image layout lines
 #'                        mtext par plot points segments strwidth text title
 #' @importFrom grDevices  colorRamp heat.colors rgb                      
@@ -9,4 +13,8 @@
 #'                        qnorm quantile rnorm runif sd var
 #' @importFrom utils      head sessionInfo
 #' @importFrom methods    hasArg
-NULL
\ No newline at end of file
+NULL
+
+"_PACKAGE"
+
+globalVariables(c('filter_constraint'))
